# w.c.s. - web application for online forms
# Copyright (C) 2005-2015  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from quixote import get_publisher, get_request, get_response, get_session, redirect
from quixote.directory import Directory
from quixote.html import TemplateIO, htmltext

from qommon.backoffice.menu import html_top
from qommon import errors, misc

from wcs.formdef import FormDef
from wcs.categories import Category
from wcs.forms.root import FormPage as PublicFormFillPage

from .management import FormBackOfficeStatusPage


class FormFillPage(PublicFormFillPage):
    def html_top(self, *args, **kwargs):
        return html_top('submission', *args, **kwargs)

    def check_role(self):
        if not self.formdef.backoffice_submission_roles:
            raise errors.AccessUnauthorizedError()
        for role in get_request().user.roles or []:
            if role in self.formdef.backoffice_submission_roles:
                break
        else:
            raise errors.AccessUnauthorizedError()

    def get_sidebar(self, data):
        r = TemplateIO(html=True)

        formdata = None
        draft_formdata_id = data.get('draft_formdata_id')
        if draft_formdata_id:
            formdata = self.formdef.data_class().get(draft_formdata_id)
        if self.formdef.enable_tracking_codes:
            r += htmltext('<h3>%s</h3>') % _('Tracking Code')
            if formdata and formdata.tracking_code:
                tracking_code = formdata.tracking_code
                r += htmltext('<p>%s</p>') % formdata.tracking_code
            else:
                r += htmltext('<p>-</p>')

        if formdata and formdata.submission_context:
            r += FormBackOfficeStatusPage(self.formdef, formdata).get_extra_context_bar()

        return r.getvalue()

    def form_side(self, step_no, page_no=0, log_detail=None, data=None, editing=None):
        r = TemplateIO(html=True)
        get_response().filter['sidebar'] = self.get_sidebar(data)
        r += htmltext('<div id="side">')
        r += self.step(step_no, page_no, log_detail, data=data, editing=editing)
        r += htmltext('</div> <!-- #side -->')
        return r.getvalue()

    def submitted(self, form, *args):
        filled = self.get_current_draft() or self.formdef.data_class()()
        filled.just_created()
        filled.data = self.formdef.get_data(form)
        filled.backoffice_submission = True
        filled.store()

        self.set_tracking_code(filled)
        get_session().remove_magictoken(get_request().form.get('magictoken'))

        url = filled.perform_workflow()
        if not url:
            url = filled.get_url(backoffice=True)

        if not self.formdef.is_of_concern_for_user(self.user, filled):
            # if the agent is not allowed to see the submitted formdef,
            # redirect to the submission homepage
            return redirect(get_publisher().get_backoffice_url() + '/submission/')

        return redirect(url)

    def save_draft(self, data, page_no):
        formdata = super(FormFillPage, self).save_draft(data, page_no)
        formdata.user_id = None
        formdata.backoffice_submission = True
        formdata.store()
        return formdata


class SubmissionDirectory(Directory):
    _q_exports = ['']

    def is_accessible(self, user):
        if not user.can_go_in_backoffice():
            return False
        # check user has at least one role set for backoffice submission
        for role_id in (user.roles or []):
            ids = FormDef.get_ids_with_indexed_value('backoffice_submission_roles', role_id)
            if ids:
                return True
        return False

    def _q_index(self):
        get_response().breadcrumb.append(('submission/', _('Submission')))
        html_top('submission', _('Submission'))
        user = get_request().user

        list_forms = []
        for formdef in FormDef.select(order_by='name', ignore_errors=True):
            if formdef.is_disabled():
                continue
            if not formdef.backoffice_submission_roles:
                continue
            for role in user.roles or []:
                if role in formdef.backoffice_submission_roles:
                    break
            else:
                continue
            list_forms.append(formdef)

        r = TemplateIO(html=True)

        cats = Category.select()
        Category.sort_by_position(cats)
        one = False
        for c in cats:
            l2 = [x for x in list_forms if str(x.category_id) == str(c.id)]
            if l2:
                r += self.form_list(l2, title=c.name)
                one = True

        l2 = [x for x in list_forms if not x.category]
        if l2:
            if one:
                title = _('Misc')
            else:
                title = None
            r += self.form_list(l2, title=title)

        return r.getvalue()

    def form_list(self, formdefs, title=None):
        r = TemplateIO(html=True)
        r += htmltext('<div class="bo-block">')
        if title:
            r += htmltext('<h2>%s</h2>') % title

        r += htmltext('<ul class="biglist">')
        for formdef in formdefs:
            r += htmltext('<li>')
            r += htmltext('<strong class="label"><a href="%s/">%s</a></strong>') % (
                    formdef.url_name, formdef.name)
            r += htmltext('</li>')
            data_class = formdef.data_class()
            formdata_ids = data_class.get_ids_with_indexed_value('status', 'draft')
            formdatas = [x for x in data_class.get_ids(formdata_ids)
                    if x.backoffice_submission is True]
            for formdata in formdatas:
                r += htmltext('<li class="smallitem">')
                label = _('Draft #%s, %s') % (formdata.id,
                        misc.localstrftime(formdata.receipt_time))
                r += htmltext('<a href="%s/%s">%s</a>') % (
                        formdef.url_name, formdata.id, label)
                r += htmltext('</li>')

        r += htmltext('</ul>')
        r += htmltext('</div>')
        return r.getvalue()

    def _q_lookup(self, component):
        get_response().breadcrumb.append(('submission/', _('Submission')))
        return FormFillPage(component)
